/***************************************************************************
                          filesocket.cpp  -  description
                             -------------------
    begin                : Thu Jan 3 2002
    copyright            : (C) 2002 by Emanuel Schmieg
    email                : kerneld@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "filesocket.h"
#include <kapp.h>
#include <iostream.h>
#include <kmessagebox.h>

FileSocket::FileSocket(int port, QString server, QFile* f, Mode transMode, int fnumber,
            QObject *parent, const char *name ) : QSocket(parent,name) {
 file = f;
 transferMode = transMode;
 filenumber = fnumber;
 if (transferMode == Upload)
   bytes = file->size();
 else
   bytes = 0;
 connect(this, SIGNAL(connected()), this, SLOT(connectedServer()));
 connect(this, SIGNAL(error(int)), this, SLOT(failed(int)));
 connect(this, SIGNAL(readyRead()), this, SLOT(readSocket()));
 connect(this, SIGNAL(connectionClosed()), this, SLOT(deleteMyself()));
 this->connectToHost(server, port);
}
FileSocket::~FileSocket(){
}
/** No descriptions */
void FileSocket::failed(int error){
 errorHandler(error, false);
}
/** No descriptions */
void FileSocket::connectedServer(){
 if (transferMode == Upload)
   sendData(QString("@put %1").arg(file->name().right(file->name().length() - (file->name().findRev('/') + 1))));
 else
   sendData(QString("@get %1").arg(filenumber));
}
/** No descriptions */
void FileSocket::readSocket(){
 while (canReadLine() && ((bytes == 0) || (transferMode == Upload)))
   {
    QString command = readLine();
    command.truncate(command.length() - 1);
    if (transferMode == Upload)
      {
       if (command == QString ("@allowed"))
         {
          sendData(QString("@bytes %1").arg(bytes));
          file->open(IO_ReadOnly);
          packets = (bytes / 1024) + 1;
          fullsteps = packets;
          pDialog = new QProgressDialog(
              i18n("Uploading %1").arg(file->name().right(file->name().length()
              - (file->name().findRev('/') + 1))), i18n("Cancel"), fullsteps, 0, i18n("Upload"), false);
          startTimer(0);
         }
       else if (command == QString ("@error"))
         errorHandler(ServerError, true);
       else if (command == QString ("@forbidden"))
         errorHandler(Refused, true);
       else
         errorHandler(Unknown, true);
      }
    else
      {
       if (command.left(command.find(' ')) == QString("@bytes"))
         {
          bytes = command.right(command.length() - (command.find(' ') + 1)).toInt();
          file->open(IO_WriteOnly);
          packets = (bytes / 1024) + 1;
          fullsteps = packets;
          pDialog = new QProgressDialog(
            i18n("Downloading %1").arg(file->name().right(file->name().length()
            - (file->name().findRev('/') + 1))), i18n("Cancel"), fullsteps, 0, i18n("Download"), false);
          startTimer(0);
         }
       else if (command == QString ("@forbidden"))
         errorHandler(Refused, true);
       else
         errorHandler(Unknown, true);
      }
   }
}
/** No descriptions */
void FileSocket::sendData(QString data){
 data += '\n';
 this->writeBlock(data.ascii(), data.length());
}
/** No descriptions */
void FileSocket::errorHandler(int error, bool mine){
 QString errorMsg;
 if (!mine)
   switch (error)
     {
      case QSocket::ErrConnectionRefused: KMessageBox::error (0, i18n ("Server refused Connection\n"
                                             "Maybe that the server does not support file handling!"));
                                          break;
      case QSocket::ErrHostNotFound     : KMessageBox::error (0, i18n ("Server not reachable\n"
                                             "Maybe the connection was interruptet?"));
                                          break;
      case QSocket::ErrSocketRead       : KMessageBox::error (0,
                                             i18n ("Error while reading socket\n"));
                                          break;
      default                           : break;
     }

 else
   switch (error)
     {
      case ServerError : KMessageBox::error(0, i18n("The server could not open the File!"));
                         break;
      case Refused     : KMessageBox::sorry(0, i18n("Sorry,\n"
                                                       "The server would not load this file up"));
                         break;
      case Unknown     : KMessageBox::error(0, i18n("Protokoll error:\n"
                                                       "Unknown data!"));
                         break;
      default          : break;
     }
 QTimer::singleShot(0, this, SLOT(deleteMyself()));
}
/** delete the socket not beautifull but usefull */
void FileSocket::deleteMyself(){
 delete this;
}
/** No descriptions */
void FileSocket::timerEvent(QTimerEvent*){
// cerr << "next KB\nbytes: " << bytes << "\npackets: " << packets << endl;
 while ((bytes - (packets-1)*1024) &&
  (((bytesAvailable() > 0) && (transferMode == Download))
  ||((!file->atEnd()) && (transferMode == Upload))))
   {
    if (transferMode == Upload)
      this->putch(file->getch());
    else
      file->putch(this->getch());
    bytes--;
   }
 if (packets)
   {
    pDialog->setProgress(fullsteps - packets);
    if (pDialog->wasCancelled())
      QTimer::singleShot(0, this, SLOT(deleteMyself()));
   }
 else
   {
    delete pDialog;
    killTimers();
    file->close();
    if (transferMode == Download)
      this->close();
   }
 if (!(bytes - (packets-1)*1024))
   packets--;
}
